<?php

namespace EventPortal\Tests\Feature\Admin\Team;

use EventPortal\Tests\TestCase;
use EventPortal\Domain\Team\Models\Team;
use EventPortal\Domain\User\Models\User;
use Illuminate\Foundation\Testing\DatabaseMigrations;

class TeamStoreTest extends TestCase
{
    use DatabaseMigrations;

    /** @test */
    public function a_team_can_be_created()
    {
        $user = factory(User::class)->create();

        $this
            ->actingAs($user)
            ->followingRedirects()
            ->post(route('admin.team.store'), [
                'name' => 'New Team',
                'description' => 'New team description',
            ])
            ->assertOk();

        $this->assertDatabaseHas('teams', [
            'name' => 'New Team',
            'description' => 'New team description',
        ]);
    }

    /** @test **/
    public function the_team_name_is_required()
    {
        $user = factory(User::class)->create();

        $this
            ->actingAs($user)
            ->post(route('admin.team.store'), [
                'name' => null,
                'description' => 'No name',
            ])
            ->assertSessionHasErrors([
                'name',
            ]);

        $this->assertDatabaseMissing('teams', [
            'description' => 'No name',
        ]);
    }

    /** @test **/
    public function the_slug_can_be_generated_automatically()
    {
        $user = factory(User::class)->create();

        $this
            ->actingAs($user)
            ->followingRedirects()
            ->post(route('admin.team.store'), [
                'name' => 'New Team',
                'slug' => null,
            ])
            ->assertOk();

        $this->assertDatabaseHas('teams', [
            'name' => 'New Team',
            'slug' => 'new-team'
        ]);
    }

    /** @test **/
    public function members_can_be_added_when_creating_a_team()
    {
        $user = factory(User::class)->create();

        list($teamMember1, $teamMember2) = factory(User::class, 2)->create();

        $this
            ->actingAs($user)
            ->followingRedirects()
            ->post(route('admin.team.store'), [
                'name' => 'New Team',
                'users' => [
                    $teamMember1->id,
                    $teamMember2->id,
                ]
            ]);

        $team = Team::first();
        $teamUserIds = $team->users->pluck('id');

        $this->assertCount(2, $team->users);
        $this->assertContains($teamMember1->id, $teamUserIds);
        $this->assertContains($teamMember2->id, $teamUserIds);
        $this->assertNotContains($user->id, $teamUserIds);
    }

    /** @test **/
    public function only_valid_members_can_be_added()
    {
        $user = factory(User::class)->create();

        $this
            ->actingAs($user)
            ->post(route('admin.team.store'), [
                'name' => 'New Team',
                'users' => [
                    321,
                    123,
                ]
            ])
            ->assertSessionHasErrors([
                'users',
            ]);

        $this->assertDatabaseMissing('teams', [
            'name' => 'New Team',
        ]);
    }
}
