<?php

namespace EventPortal\Tests\Feature\Admin\Team;

use EventPortal\Tests\TestCase;
use EventPortal\Domain\Team\Models\Team;
use EventPortal\Domain\User\Models\User;
use Illuminate\Foundation\Testing\DatabaseMigrations;

class TeamUpdateTest extends TestCase
{
    use DatabaseMigrations;

    /** @test */
    public function a_team_can_be_updated()
    {
        $user = factory(User::class)->create();
        $team = factory(Team::class)->create([
            'name' => 'Original Name',
            'description' => 'Original description',
        ]);

        $this
            ->actingAs($user)
            ->followingRedirects()
            ->withoutExceptionHandling()
            ->post(route('admin.team.update', $team), [
                'name' => 'New Name',
                'description' => 'New description',
            ])
            ->assertOk();

        $this->assertDatabaseHas('teams', [
            'name' => 'New Name',
            'description' => 'New description',
        ]);
    }

    /** @test **/
    public function the_team_name_is_required()
    {
        $user = factory(User::class)->create();
        $team = factory(Team::class)->create([
            'name' => 'Original Name',
        ]);

        $this
            ->actingAs($user)
            ->post(route('admin.team.update', $team), [
                'name' => null,
                'description' => 'No name',
            ])
            ->assertSessionHasErrors([
                'name',
            ]);

        $this->assertDatabaseHas('teams', [
            'name' => 'Original Name',
        ]);

        $this->assertDatabaseMissing('teams', [
            'description' => 'No name',
        ]);
    }

    /** @test **/
    public function members_can_be_updated()
    {
        $user = factory(User::class)->create();
        $team = factory(Team::class)->create([
            'name' => 'Original Name',
        ]);
        list($teamMember1, $teamMember2) = factory(User::class, 2)->create();

        $team->users()->sync($teamMember1->id);

        $this
            ->actingAs($user)
            ->followingRedirects()
            ->post(route('admin.team.update', $team), [
                'name' => 'New Team',
                'users' => [
                    $teamMember1->id,
                ]
            ]);

        $team = Team::first();
        $teamUserIds = $team->users->pluck('id');

        $this->assertCount(1, $team->users);
        $this->assertContains($teamMember1->id, $teamUserIds);
        $this->assertNotContains($teamMember2->id, $teamUserIds);
        $this->assertNotContains($user->id, $teamUserIds);
    }

    /** @test **/
    public function only_valid_members_can_be_added()
    {
        $user = factory(User::class)->create();
        $team = factory(Team::class)->create([
            'name' => 'Original Name',
        ]);
        list($teamMember1, $teamMember2) = factory(User::class, 2)->create();

        $team->users()->sync([$teamMember1->id, $teamMember2->id]);

        $this
            ->actingAs($user)
            ->post(route('admin.team.store'), [
                'name' => 'New Team',
                'users' => [
                    321,
                    123,
                ]
            ])
            ->assertSessionHasErrors([
                'users',
            ]);

        $team = Team::first();
        $teamUserIds = $team->users->pluck('id');

        $this->assertCount(2, $team->users);
        $this->assertContains($teamMember1->id, $teamUserIds);
        $this->assertContains($teamMember2->id, $teamUserIds);
        $this->assertNotContains($user->id, $teamUserIds);
    }
}
