<?php

namespace EventPortal\Tests\Feature\Admin\Team;

use Carbon\Carbon;
use EventPortal\Domain\Team\Models\Team;
use EventPortal\Domain\User\Models\User;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Spatie\Snapshots\MatchesSnapshots;

class UserStoreTest extends TestCase
{
    use DatabaseMigrations, MatchesSnapshots;

    /** @test **/
    public function the_create_view_supplies_the_correct_data_to_the_frontend()
    {
        $user = factory(User::class)->create();
        factory(Team::class)->state('snapshot')->create();

        $this->assertMatchesJsonSnapshot(
            $this->actingAs($user)
                ->get(route('admin.user.create'))->props()
        );
    }

    /** @test */
    public function a_user_can_be_created()
    {
        $admin = factory(User::class)->create();

        $this
            ->actingAs($admin)
            ->followingRedirects()
            ->post(route('admin.user.store'), [
                'first_name' => 'Firstname',
                'last_name' => 'Lastname',
            ])
            ->assertOk();

        $this->assertDatabaseHas('users', [
            'first_name' => 'Firstname',
            'last_name' => 'Lastname',
        ]);
    }

    /** @test **/
    public function the_first_name_last_name_and_email_are_required()
    {
        $admin = factory(User::class)->create();

        $this
            ->actingAs($admin)
            ->post(route('admin.user.store'), [
                'first_name' => null,
                'last_name' => null,
            ])
            ->assertSessionHasErrors([
                'first_name',
                'last_name',
            ]);

        $this->assertEquals(1, User::count()); // Admin user already exists
    }

    /** @test **/
    public function a_user_can_be_added_to_teams()
    {
        $admin = factory(User::class)->create();

        list($team1, $team2) = factory(Team::class, 2)->create();

        $this
            ->actingAs($admin)
            ->followingRedirects()
            ->post(route('admin.user.store'), [
                'first_name' => 'Firstname',
                'last_name' => 'Lastname',
                'teams' => [
                    $team1->id,
                    $team2->id,
                ]
            ]);

        $newUser = User::where('first_name', 'Firstname')->first();
        $newUserTeamIds = $newUser->teams->pluck('id');

        $this->assertCount(2, $newUser->teams);
        $this->assertContains($team1->id, $newUserTeamIds);
        $this->assertContains($team2->id, $newUserTeamIds);
    }

    /** @test **/
    public function only_valid_teams_can_be_added()
    {
        $admin = factory(User::class)->create();

        $this
            ->actingAs($admin)
            ->post(route('admin.user.store'), [
                'first_name' => 'Firstname',
                'last_name' => 'Lastname',
                'teams' => [
                    321,
                    123,
                ]
            ])
            ->assertSessionHasErrors([
                'teams',
            ]);

        $this->assertDatabaseMissing('teams', [
            'name' => 'New Team',
        ]);
    }
}
