<?php

namespace EventPortal\Tests\Feature\Admin\Team;

use Carbon\Carbon;
use EventPortal\Domain\Team\Models\Team;
use EventPortal\Domain\User\Models\User;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Spatie\Snapshots\MatchesSnapshots;

class UserUpdateTest extends TestCase
{
    use DatabaseMigrations, MatchesSnapshots;

    /** @test **/
    public function the_edit_view_supplies_the_correct_data_to_the_frontend()
    {
        Carbon::setTestNow('2020-07-03 16:00:00');

        $admin = factory(User::class)->create();
        $user = factory(User::class)->state('snapshot')->create();

        factory(Team::class)->state('snapshot')->create();

        $this->assertMatchesJsonSnapshot(
            $this->actingAs($admin)
                ->get(route('admin.user.edit', $user->id))->props()
        );
    }

    /** @test */
    public function a_user_can_be_updated()
    {
        $admin = factory(User::class)->create();
        $user = factory(User::class)->create([
            'first_name' => 'Original First',
            'last_name' => 'Original Last',
        ]);

        $this
            ->actingAs($admin)
            ->followingRedirects()
            ->post(route('admin.user.update', $user), array_merge($user->toArray(), [
                'first_name' => 'New First',
                'last_name' => 'New Last',
            ]))
            ->assertOk();

        $this->assertDatabaseHas('users', [
            'first_name' => 'New First',
            'last_name' => 'New Last',
        ]);
    }

    /** @test **/
    public function the_first_name_last_name_and_email_are_required()
    {
        $admin = factory(User::class)->create();
        $user = factory(User::class)->create([
            'first_name' => 'Original First',
            'last_name' => 'Original Last',
        ]);

        $this
            ->actingAs($admin)
            ->post(route('admin.user.update', $user), [
                'first_name' => null,
                'last_name' => null,
            ])
            ->assertSessionHasErrors([
                'first_name',
                'last_name',
            ]);

        $user->refresh();

        $this->assertEquals('Original First', $user->first_name);
        $this->assertEquals('Original Last', $user->last_name);
    }

    /** @test **/
    public function teams_can_be_updated()
    {
        $admin = factory(User::class)->create();
        $user = factory(User::class)->create();
        [$team1, $team2] = factory(Team::class, 2)->create();

        $user->teams()->sync($team1->id);

        $this
            ->actingAs($admin)
            ->followingRedirects()
            ->post(route('admin.user.update', $user), array_merge($user->toArray(), [
                'teams' => [
                    $team2->id,
                ],
            ]));

        $user->refresh();
        $userTeamIds = $user->teams->pluck('id');

        $this->assertCount(1, $user->teams);
        $this->assertContains($team2->id, $userTeamIds);
        $this->assertNotContains($team1->id, $userTeamIds);
    }

    /** @test **/
    public function only_valid_teams_can_be_added()
    {
        $admin = factory(User::class)->create();
        $user = factory(User::class)->create();
        [$team1, $team2] = factory(Team::class, 2)->create();

        $user->teams()->sync([
            $team1->id,
            $team2->id
        ]);

        $this
            ->actingAs($admin)
            ->post(route('admin.user.update', $user), array_merge($user->toArray(), [
                'teams' => [
                    321,
                    123,
                ]
            ]))
            ->assertSessionHasErrors([
                'teams',
            ]);

        $user->refresh();
        $userTeamIds = $user->teams->pluck('id');

        $this->assertCount(2, $user->teams);
        $this->assertContains($team1->id, $userTeamIds);
        $this->assertContains($team2->id, $userTeamIds);
    }
}
